<?php

namespace Drupal\ump\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * @file
 * Contains \Drupal\ump\Controller\UmpOverviewController.
 */

/**
 * Form controller for the UMP Overview form.
 *
 * @ingroup ump
 */
class UmpOverviewController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public function content() {

    $header = [
      $this->t("Role"),
      $this->t("Allow to create"),
      $this->t("Allow to view"),
      $this->t("Allow to edit"),
      $this->t("Allow to delete"),
      $this->t("Operations"),
    ];
    $rows = [];
    $empty_text = $this->t("No rules created yet.");

    // Get the users roles defined and the custom user fields defined.
    $get_roles = ump_get_user_roles();
    $user_fields = ump_get_user_user_fields();

    // Get the rules defined.
    $rules = ump_get_ump_crud_rules_all();
    if (is_array($rules)) {
      foreach ($rules as $rule_key) {
        $rule = \Drupal::config('ump.settings')->get($rule_key);

        $allow_to_create = "";
        if ($rule["allow_to_create_enable"] == 1) {
          $allow_to_create .= "<b>" . $get_roles[$rule["allow_to_create_role"]] . "</b> ";
          if ($rule["allow_to_create_overtake"] == 1) {
            $allow_to_create .= $this->t("and take over the field") . " <b>" . $user_fields[$rule["allow_to_create_overtake_field"]] . "</b>";
          }
        }
        $allow_to_view = "";
        if ($rule["allow_to_view_enable"] == 1) {
          $allow_to_view .= "<b>" . $get_roles[$rule["allow_to_view_role"]] . "</b> ";
          $allow_to_view .= $this->t("which") . " <b>" . $rule["allow_to_view_which"] . "</b>";
          if ($rule["allow_to_view_which"] == "if_share_common_field") {
            $allow_to_view .= " " . $this->t("field") . " <b>" . $user_fields[$rule["allow_to_view_common_field"]] . "</b>";
          }
        }
        $allow_to_edit = "";
        if ($rule["allow_to_edit_enable"] == 1) {
          $allow_to_edit .= "<b>" . $get_roles[$rule["allow_to_edit_role"]] . "</b> ";
          $allow_to_edit .= $this->t("which") . " <b>" . $rule["allow_to_edit_which"] . "</b>";
          if ($rule["allow_to_edit_which"] == "if_share_common_field") {
            $allow_to_edit .= " " . $this->t("field") . " <b>" . $user_fields[$rule["allow_to_edit_common_field"]] . "</b>";
          }
        }
        $allow_to_delete = "";
        if ($rule["allow_to_delete_enable"] == 1) {
          $allow_to_delete .= "<b>" . $get_roles[$rule["allow_to_delete_role"]] . "</b> ";
          $allow_to_delete .= $this->t("which") . " <b>" . $rule["allow_to_delete_which"] . "</b>";
          if ($rule["allow_to_delete_which"] == "if_share_common_field") {
            $allow_to_delete .= " " . $this->t("field") . " <b>" . $user_fields[$rule["allow_to_delete_common_field"]] . "</b>";
          }
        }

        $edit_url = Url::fromRoute('ump.rule.edit.form', ['rule_key' => $rule_key]);
        $edit_limk = Link::fromTextAndUrl($this->t('Edit'), $edit_url)->toRenderable();
        $delete_url = Url::fromRoute('ump.rule.delete.form', ['rule_key' => $rule_key]);
        $delete_limk = Link::fromTextAndUrl($this->t('Delete'), $delete_url)->toRenderable();
        $rows[] = [
          [
            'data' => [
              '#markup' => "<b>" . $get_roles[$rule["select_role"]] . "</b>",
            ],
          ],
          [
            'data' => [
              '#markup' => $allow_to_create,
            ],
          ],
          [
            'data' => [
              '#markup' => $allow_to_view,
            ],
          ],
          [
            'data' => [
              '#markup' => $allow_to_edit,
            ],
          ],
          [
            'data' => [
              '#markup' => $allow_to_delete,
            ],
          ],
          [
            'data' => [
              '#markup' => render($edit_limk) . " " . render($delete_limk),
            ],
          ],
        ];
      }
    }

    $add_new_url = Url::fromRoute('ump.rule.create.form');
    $add_new_limk = Link::fromTextAndUrl($this->t('Create Rule'), $add_new_url)->toRenderable();

    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#sticky' => TRUE,
      '#empty' => $empty_text,
      '#caption' => [
        'data' => [
          '#markup' => render($add_new_limk),
        ],
      ],
    ];
  }

}
