<?php

namespace Drupal\ump\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Drupal\user\Entity\Role;

/**
 * @file
 * Contains \Drupal\ump\Controller\UmpOverviewController.
 */

/**
 * Form controller for the UMP User Management form.
 *
 * @ingroup ump
 */
class UmpUserManagementController extends ControllerBase {

  /**
   * {@inheritdoc}
   */
  public function content() {
    $header = [
      $this->t("Name"),
      $this->t("Roles"),
      $this->t("Operations"),
    ];
    $rows = [];
    $empty_text = $this->t("No users found.");

    $rules = ump_get_ump_crud_rules_all();
    foreach ($rules as $rule_key) {
      $rule = \Drupal::config('ump.settings')->get($rule_key);
      if ($rule["allow_to_view_enable"] == 1) {
        $role = Role::load($rule["allow_to_view_role"]);

        // Get the users that we are allowed to be displayed.
        $uids = \Drupal::entityQuery('user')
          ->condition('roles', $role->id())
          ->execute();
        $users = User::loadMultiple($uids);

        foreach ($users as $load_user) {
          // Check the view permission.
          if (\Drupal::service('path.validator')->isValid('user/' . $load_user->id())) {
            $roles = "";
            $edit = "";
            $delete = "";
            // Check the edit permission and create the edit link.
            if (\Drupal::service('path.validator')->isValid('user/' . $load_user->id() . '/edit')) {
              $edit_url = Url::fromRoute('entity.user.edit_form', ['user' => $load_user->id()]);
              $edit_limk = Link::fromTextAndUrl($this->t('Edit'), $edit_url)->toRenderable();
              $edit = render($edit_limk);
            }
            // Check the delete permission and create the delete link.
            if (\Drupal::service('path.validator')->isValid('user/' . $load_user->id() . '/cancel')) {
              $delete_url = Url::fromRoute('entity.user.cancel_form', ['user' => $load_user->id()]);
              $delete_limk = Link::fromTextAndUrl($this->t('Delete'), $delete_url)->toRenderable();
              $delete = render($delete_limk);
            }

            foreach ($load_user->getRoles() as $load_user_value) {
              $roles .= $load_user_value . "<br>";
            }

            $rows[] = [
              [
                'data' => [
                  '#markup' => $load_user->getAccountName(),
                ],
              ],
              [
                'data' => [
                  '#markup' => $roles,
                ],
              ],
              [
                'data' => [
                  '#markup' => $edit . " " . $delete,
                ],
              ],
            ];
          }
        }
      }
    }

    $add_new_limk = '';
    if (\Drupal::service('path.validator')->isValid('admin/people/create')) {
      $add_new_url = Url::fromRoute('user.admin_create');
      $add_new_limk = Link::fromTextAndUrl($this->t('Create user'), $add_new_url)->toRenderable();
    }

    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#sticky' => TRUE,
      '#empty' => $empty_text,
      '#caption' => [
        'data' => [
          '#markup' => render($add_new_limk),
        ],
      ],
    ];
  }

}
